#include "StdAfx.h"
#include "WASCommand.h"
#include "WASActor.h"
#include "UtilFunction.h"
#include "MessengerActor.h"
#include "ConvertUTF.h"
#include "GameTalkAPIGlobal.h"
#include "UtilFunction.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CWASCommand::CWASCommand(CWASActor *pActor) :
	m_eError(wasError_None)
{
	memset(&m_sCacheResult, 0, sizeof(m_sCacheResult));
	m_nWASError = -1;
	m_pActor = pActor;
}

CWASCommand::~CWASCommand(void)
{
//	DLOG("CWASCommand::~CWASCommand(void)\n");
    
	m_pActor->DeleteCacheResult(GetID());
}


void CWASCommand::CompleteCommand(void) 
{ 
	m_httpFacade.SetWorking(false); 

	m_pActor->DeleteCacheResult(GetID());
    
	//m_httpFacade.m_log.Trace("GBL <R-%04d> %s : RESPONSE = %d(%d)", GetID().GetID(), m_httpFacade.session().GetURL(), m_eError, m_nWASError);
}


bool CWASCommand::GetProgressInfo(size_t &totalSize, size_t &currentSize)
{
    return m_httpFacade.GetProgressInfo(totalSize, currentSize);
}
       

void CWASCommand::OnHttpCompleted(CHttpFacade * pHttpFacade, bool bSuccess, bool bCancel)
{
	std::string strResultCode;

	if( !bSuccess )
	{
		 StringUtil::StringFormat<WSTRING>(m_strWasErrMsg, _T("CHttpSession fail, wininet error code=%d"), 
			pHttpFacade->session().GetInternalLastError());
		if(bCancel)
			m_eError = wasError_Canceled;
		else
			m_eError = wasError_ConnectFail;

		CallWasResponse(false, pHttpFacade, strResultCode, wasResDataType_Unknown, NULL);

		return;
	}

	OutputDebugStringA( (LPCSTR)pHttpFacade->session().GetFileData() );
	
	nfc::CIkXMLDom dom;			
	char *szResponseData = (char *)pHttpFacade->session().GetFileData();
	DWORD nResponseDataLen = pHttpFacade->session().GetFileSize();

	if( dom.loadXML( szResponseData, nResponseDataLen ) )
	{
		::OutputDebugString(L"CWASCommand::OnHttpCompleted() XML DATA\n");
		CallWasResponse(true, pHttpFacade, strResultCode, wasResDataType_XML, &dom);
		return;
	}

	Json::Value jsonValue;
	Json::Reader jsonReader;

	if( jsonReader.parse( szResponseData, jsonValue ) )
	{
		::OutputDebugString(L"CWASCommand::OnHttpCompleted() JSON DATA\n");
		CallWasResponse(true, pHttpFacade, strResultCode, wasResDataType_JSON, &jsonValue);
		return;
	}

	USES_CONVERSION;

	m_strWasErrMsg = _T("load XML or parse JSON fail!!\r\n");
	m_eError = wasError_Protocol;

	WSTRING strTmp;
	StringUtil::StringFormat<WSTRING>(strTmp,  _T("HTTP header \"nphone-resultcode:\" = [%s], http response code=%d"),
		A2CT(pHttpFacade->m_strNPhoneResultCode.c_str()),
		pHttpFacade->session().GetStatusCode() );
	m_strWasErrMsg += strTmp;

	::OutputDebugString(L"CWASCommand::OnHttpCompleted() UNAVAILABLE DATA FORMAT!!!\n");

	CallWasResponse(false, pHttpFacade, strResultCode, wasResDataType_Unknown, NULL);
}

void CWASCommand::Call_OnWasResponse(trid_t trid, bool bSuccess, int nWinINetErrorCode, int nHttpResponseCode, LPCSTR szWasResCodeInHeader, LPCSTR szWasResCodeInXml, NTWASResDataType resDataType, void * pData)
{
    // LHC_ELOG
    static bool s_firstTime = true;
    if ( s_firstTime )
	{
        s_firstTime = false;

//        if ( bSuccess )
//            GT_ELOG_NOARG( L"conn", L"{\"svr\":\"was\",\"act\":\"request\",\"res\":\"ok\"}" );
//        else
//            GT_ELOG( L"conn", L"{\"svr\":\"was\",\"act\":\"request\",\"res\":\"fail\",\"err\":%d}", nWinINetErrorCode );
    }

	if( wasResDataType_XML == resDataType )
		OnWasResponse( trid, bSuccess, nWinINetErrorCode, nHttpResponseCode, szWasResCodeInHeader, szWasResCodeInXml, (nfc::CIkXMLDom *)pData );
	else if( wasResDataType_JSON == resDataType )
		OnWasResponse( trid, bSuccess, nWinINetErrorCode, nHttpResponseCode, szWasResCodeInHeader, szWasResCodeInXml, (Json::Value *)pData );
	else
	{
		OnWasResponse( trid, bSuccess, 0, 0, "", szWasResCodeInXml, (nfc::CIkXMLDom *)pData );

		if( m_httpFacade.m_trid < 0 )
			return;

		OnWasResponse( trid, bSuccess, 0, 0, "", szWasResCodeInXml, (Json::Value *)pData );
	}
}

void CWASCommand::SetErrorCode( int nCode )
{
	switch(nCode)
	{
	case 100: // 
		m_eError = wasError_None;
		break;
	case 201:
		m_eError = wasError_InvalidParameter;
		break;
	case 202:
		m_eError = wasError_Auth;
		break;
	case 203:
		m_eError = wasError_DB;
		break;
	case 206:
		m_eError = wasError_NotEmpty;
		break;
	case 208:
		m_eError = wasError_Max;
		break;
	case 210:
		m_eError = wasError_TabooWord;
		break;
	case 211:	// SERVER_DOWN
		m_eError = wasError_ServerDown;
		break;
	case 212:
		m_eError = wasError_PenaltyUser;
		break;
	case 213:
		m_eError = wasError_NotFound;
		break;
	case 214:
		m_eError = wasError_NotAffectedRow;
		break;
	case 215:
		m_eError = wasError_DeletedData;
		break;
	case 216:
		m_eError = wasError_PenaltyRoom;
		break;
	case 217:
		m_eError = wasError_NotActiveData;
		break;
	case 218:
		m_eError = wasError_RoomInUse;
		break;
	case 220:
		m_eError = wasError_AuthTicketExpired;
		break;
	case 444:	// ROOM SERVER DOWN
		m_eError = wasError_ServerDown;
		break;
	case 300:
		m_eError = wasError_Invalid_Authorization;
		break;
	case 801:
		m_eError = wasError_Lack_Point;
		break;

	default:
		if(nCode > 1000)
			m_eError = wasError_DB;
		else
			m_eError = wasError_Etc;
		break;
	}
}

void CWASCommand::CallWasResponse(bool bSuccess,  CHttpFacade * pHttpFacade, std::string &strResultCode, NTWASResDataType resDataType, void *pResponseData )
{
	if( bSuccess && wasResDataType_XML == resDataType )
	{
		nfc::CIkXMLDom *pdom = (nfc::CIkXMLDom *)pResponseData;

		m_strResultXMLBuffer.assign((LPCSTR)pHttpFacade->session().GetFileData(), pHttpFacade->session().GetFileSize());

		nfc::IIkXMLNode *pNode = NULL;
		nfc::IIkXMLAttribute *pAttr = NULL;

		//   Ľ
		pNode = pdom->GetdocumentElement()->selectSingleNode("result");
		if( pNode == NULL )
		{
			if( pHttpFacade->m_strNPhoneResultCode != "100" )
			{
				m_strWasErrMsg = _T("cannot find <result> node");
				m_eError = wasError_Protocol;
				bSuccess = false;
			}
		}
		else
		{
			pAttr = pNode->getAttribute("code");
			if( pAttr == NULL )
			{
				m_strWasErrMsg = _T("cannot find <result>::code attribute");
				bSuccess = false;
			}
			else
			{
				USES_UTF8_CONVERSION;
				strResultCode = pAttr->Gettext().c_str();
				pAttr = pNode->getAttribute("msg");
				if( pAttr )
					m_strResultMsg = U82W(pAttr->Gettext().c_str());

				int nCode = atoi( strResultCode.c_str() );

				m_nWASError = nCode;
				SetErrorCode(nCode);

				if(m_eError != wasError_None )
				{
					StringUtil::StringFormat<WSTRING>(m_strWasErrMsg, _T("%s[%d]"), m_strResultMsg.c_str(), nCode);
					bSuccess = false;
				}
			}
		}
	}

	Call_OnWasResponse(GetID(), bSuccess, pHttpFacade->session().GetInternalLastError(), pHttpFacade->session().GetStatusCode(),
		pHttpFacade->m_strNPhoneResultCode.c_str(), strResultCode.c_str(), resDataType, pResponseData);
}

